// NSSplitter.cpp : implementation file
//

#include "stdafx.h"
#include "NSSplitter.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CNSSplitter

IMPLEMENT_DYNCREATE(CNSSplitter, CSplitterWnd)

CNSSplitter::CNSSplitter()
{
	// default splitter box/bar sizes (includes borders)
	m_szImage=CSize(0,0);
	m_rcLastImageRect=CRect(0,0,0,0);
	m_nSplitterWidth= 7;
	m_bFlatSplitter=FALSE;
	SetAsFlat(TRUE);
	
	m_bHoverBitmapDrawn=m_bUseHandCursor=FALSE;
	hCursorArrow=hCursorHand=NULL;

	m_nOptimizedSplitterSize =200;
	nTimerID=0;

}

CNSSplitter::~CNSSplitter()
{
	if (hCursorHand)
		::DestroyCursor(hCursorHand);
	if (hCursorArrow)
		::DestroyCursor(hCursorArrow);
}

BEGIN_MESSAGE_MAP(CNSSplitter, CSplitterWnd)
	//{{AFX_MSG_MAP(CNSSplitter)
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONDBLCLK()
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CNSSplitter message handlers
int CNSSplitter::SetSplitterWidth(int nWidth)
{
	int nPrevWidth = m_nSplitterWidth;
	m_nSplitterWidth = nWidth;
	return nPrevWidth;
}

BOOL CNSSplitter::UseHandCursor(BOOL bHand)
{
	BOOL bPrev = m_bUseHandCursor;
	m_bUseHandCursor = bHand;
	return bPrev;
}

BOOL CNSSplitter::LoadCursors()
{
	if (!hCursorArrow)
		hCursorArrow = ::LoadCursor(NULL,MAKEINTRESOURCE(IDC_ARROW));
	if (!hCursorHand)
	{
		CString windir;
		::GetWindowsDirectory(windir.GetBuffer(MAX_PATH), MAX_PATH);
		windir.ReleaseBuffer();
		windir += _T("\\winhlp32.exe");
		HMODULE hModule = LoadLibrary(windir);
		if (hModule){
			hCursorHand = CopyCursor(::LoadCursor(hModule, MAKEINTRESOURCE(106)));
			FreeLibrary(hModule);
			return TRUE;
		}
	}

	// status of both cursors
	return (hCursorHand!=NULL && hCursorArrow!=NULL);
}

int CNSSplitter::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CSplitterWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	LoadCursors(); // load custom cursors (hand+arrow)
	// create timer
	nTimerID = SetTimer(1000,200,NULL);
	return 0;
}

int CNSSplitter::SetOptimizedSize(int cx,BOOL bFromLeftTop)
{
	int nPrev = m_nOptimizedSplitterSize;
	if (cx>0) {
		cx = (bFromLeftTop) ? cx : -1*cx;
		m_nOptimizedSplitterSize = cx;
	}
	return nPrev;
}
// use custom cursors : hand or arrow
BOOL CNSSplitter::SetCustomCursor(BOOL bHandCursor)
{
	return (bHandCursor) ?
		(BOOL)::SetCursor(hCursorHand) : (BOOL)::SetCursor(hCursorArrow);
}

/*	Both functions track whether mouse is on image in oreder to reset the 
cursor and repaint the image (for un/hovereffect) */
BOOL CNSSplitter::IsMouseOnBitmap()
{
	CPoint point(GetMessagePos());
	ScreenToClient(&point);
	return IsMouseOnBitmap(point);
}
BOOL CNSSplitter::IsMouseOnBitmap(CPoint pt)
{ 
	static CRect rc;
	rc = m_rcLastImageRect;
	int nAdd = (IsFlat()) ? 2 : 1;
	if (IsVerticalSplitter())
		rc.InflateRect(nAdd,0,nAdd,0);
	else  rc.InflateRect(0,nAdd,0,nAdd);
	return rc.PtInRect(pt);
}

/* Treat splitter as flat : when using this function after
splitter was shown : (visible) you need to redraw the whole splitter
(with frame) so internal CSplitterWnd variables will take effect! */
BOOL CNSSplitter::SetAsFlat(BOOL bFlat)
{
	BOOL bPrev = m_bFlatSplitter;

	// NOTICE : these are non documented variables whithis CSplitterWnd class
	m_cxSplitter = m_cySplitter = m_nSplitterWidth + 2 + 2;
	m_cxBorderShare = m_cyBorderShare = 0;
	m_cxSplitterGap = m_cySplitterGap = m_nSplitterWidth + 2 + 2;

	if (!bFlat) // edge parameters (not flat)
	{
		// borders of non flat are wider than flat
		m_cxBorder = m_cyBorder = 2;
	} else { // flat parameters
		m_cxBorder = m_cyBorder = 1;
	}

#ifdef _DEBUG
	if (GetSystemMetrics(SM_CXBORDER) != 1 ||
		GetSystemMetrics(SM_CYBORDER) != 1)
	{
		TRACE0("Warning: CSplitterWnd assumes 1 pixel border.\n");
		// will look ugly if borders are not 1 pixel wide and 1 pixel high
	}
#endif

	m_bFlatSplitter = bFlat;
	if (bPrev!=m_bFlatSplitter && ::IsWindow(m_hWnd))
		RecalcLayout();

	return bPrev;
}

/* Multi purpose function to determine if bar on end of splitter window
: used when double clicked : to determine new position , etc. */
BOOL CNSSplitter::IsBarOnEdge()
{
	if (IsVerticalSplitter() && m_pColInfo[0].nCurSize==0)
		return TRUE;
	if (!IsVerticalSplitter() && m_pRowInfo[0].nCurSize==0)
		return TRUE;
	return FALSE;
}
/* Still non used function , useful to use when you want to keep previous
splitter size and position : to return to previous size */
void CNSSplitter::SetLastSize()
{
	if (IsVerticalSplitter())
		m_nOptimizedSplitterSize = (m_pColInfo) ? m_pColInfo[0].nCurSize : 0;
	else // horizontal splitter
		m_nOptimizedSplitterSize = (m_pRowInfo) ? m_pRowInfo[0].nCurSize : 0;
}


#define SMALLDIFF 2
#define BIGDIFF 4
#define ARROWBASE 8
#define ARROWHEIGHT 4

/* This function draw -primitive- triangles at the edges of the bitmap
(8 pixels from top/bottom in case of verticle bar, 8 pixels from left/right
in case of horizontal bitmap */

void CNSSplitter::CreateArrowRgn(CRgn &rgn,const CRect &rect,
		BOOL bVerticalSplitter,
		BOOL bDirectionLeftTop,BOOL bFirstArrow)
{
	static CPoint ptVert[3];
	static CPoint ptHorz[3];
	static int i;
/*  These are default shapes :

	*
	* *				  * * * * * * *
	*   *				*       *
	*     *		          *   *
	*   *			        *
	* *
	*                                   */

	if (bVerticalSplitter) { // Vertical splitter
		if (bFirstArrow) {
			if (bDirectionLeftTop) {
				ptVert[0] = CPoint(rect.left+SMALLDIFF+ARROWHEIGHT,rect.top+BIGDIFF);  
				ptVert[1] = CPoint(rect.left+SMALLDIFF+ARROWHEIGHT,rect.top+BIGDIFF+ARROWBASE);  
				ptVert[2] = CPoint(rect.left+SMALLDIFF,rect.top+BIGDIFF+ARROWBASE/2);
			} else {
				ptVert[0] = CPoint(rect.left+SMALLDIFF,rect.top+BIGDIFF);  
				ptVert[1] = CPoint(rect.left+SMALLDIFF,rect.top+BIGDIFF+ARROWBASE);  
				ptVert[2] = CPoint(rect.left+SMALLDIFF+ARROWHEIGHT,rect.top+BIGDIFF+ARROWBASE/2);
			}
		} else {
			if (bDirectionLeftTop) {
				ptVert[0] = CPoint(rect.left+SMALLDIFF+ARROWHEIGHT,rect.bottom-BIGDIFF-ARROWBASE);  
				ptVert[1] = CPoint(rect.left+SMALLDIFF+ARROWHEIGHT,rect.bottom-BIGDIFF);  
				ptVert[2] = CPoint(rect.left+SMALLDIFF,rect.bottom-BIGDIFF-ARROWBASE/2);
			} else {
				ptVert[0] = CPoint(rect.left+SMALLDIFF,rect.bottom-BIGDIFF-ARROWBASE);  
				ptVert[1] = CPoint(rect.left+SMALLDIFF,rect.bottom-BIGDIFF);  
				ptVert[2] = CPoint(rect.left+SMALLDIFF+ARROWHEIGHT,rect.bottom-BIGDIFF-ARROWBASE/2);
			}
		}
		rgn.CreatePolygonRgn(ptVert,3,ALTERNATE); //WINDING);//
	} 
	else { // Horizontal splitter
		if (bFirstArrow) {
			if (bDirectionLeftTop) {
				ptHorz[0] = CPoint(rect.left+BIGDIFF,rect.top+SMALLDIFF+ARROWHEIGHT);
				ptHorz[1] = CPoint(rect.left+BIGDIFF+ARROWBASE,rect.top+SMALLDIFF+ARROWHEIGHT);
				ptHorz[2] = CPoint(rect.left+BIGDIFF+ARROWBASE/2,rect.top+SMALLDIFF);
			} else {
				ptHorz[0] = CPoint(rect.left+BIGDIFF,rect.top+SMALLDIFF);
				ptHorz[1] = CPoint(rect.left+BIGDIFF+ARROWBASE,rect.top+SMALLDIFF);
				ptHorz[2] = CPoint(rect.left+BIGDIFF+ARROWBASE/2,rect.top+SMALLDIFF+ARROWHEIGHT);
			}
		} else {
			if (bDirectionLeftTop) {
				ptHorz[0] = CPoint(rect.right-BIGDIFF-ARROWBASE,rect.top+SMALLDIFF+ARROWHEIGHT);
				ptHorz[1] = CPoint(rect.right-BIGDIFF,rect.top+SMALLDIFF+ARROWHEIGHT);
				ptHorz[2] = CPoint(rect.right-BIGDIFF-ARROWBASE/2,rect.top+SMALLDIFF);
			} else {
				ptHorz[0] = CPoint(rect.right-BIGDIFF-ARROWBASE,rect.top+SMALLDIFF);
				ptHorz[1] = CPoint(rect.right-BIGDIFF,rect.top+SMALLDIFF);
				ptHorz[2] = CPoint(rect.right-BIGDIFF-ARROWBASE/2,rect.top+SMALLDIFF+ARROWHEIGHT);
			}
		}

		rgn.CreatePolygonRgn(ptHorz,3,ALTERNATE);// WINDING);//
	}
}

/* Called when needed to repaint bitmap on bar whenever mouse hovers or leaves 
the area of bar : doesnt use WM_MOUSEENTER/LEAVE so suitable for Win95 systems. */
void CNSSplitter::UpdateSplitterBitmap(BOOL bChecked)
{
	if (m_bTracking) return;
	BOOL bDraw=bChecked;

	BOOL bMouseHoverBitmap = IsMouseOnBitmap();
	if (!bChecked) {
		if ((bMouseHoverBitmap && !m_bHoverBitmapDrawn) ||
			(!bMouseHoverBitmap && m_bHoverBitmapDrawn)) 
		{
			bChecked=TRUE;
		}
	}

	if (bChecked)
	{
		m_bHoverBitmapDrawn = bMouseHoverBitmap;
		InvalidateRect(&m_rcLastImageRect,TRUE); //	InvalidateRect(&m_rcLastImageRect,TRUE);
		UpdateWindow();
	}
}
// The code that draws the bitmap (+ some optimization for clipped area)
void CNSSplitter::DrawSplitterBitmap(CDC* pDC, ESplitType nType, const CRect& rectArg)
{
	if (!pDC || nType!=splitBar || !::IsWindow(m_hWnd))
		return; // only split bar

	CRect rcClip;
	if (ERROR == pDC->GetClipBox(&rcClip))
		return; // device context is invalid
	CRect r = rectArg;

	BOOL bVerticalSplitter = IsVerticalSplitter();
	BOOL bMouseOnBitmap = IsMouseOnBitmap();
	BOOL bBarOnEdge = IsBarOnEdge();

	CPoint ptSplitterBarStartPoint;
	CPoint ptBitmapStartPoint=CPoint(0,0);
	CSize szBitmapToDraw = CSize(min(m_szImage.cx,rectArg.Width()),min(m_szImage.cy,rectArg.Height()));

	// vertical splitter
	ptSplitterBarStartPoint.x = (rectArg.Width()-m_szImage.cx)/2;
	ptSplitterBarStartPoint.y = (rectArg.Height()-m_szImage.cy)/2;
	if (ptSplitterBarStartPoint.x<0) {
		ptBitmapStartPoint.x=ptSplitterBarStartPoint.x*-1;
		ptSplitterBarStartPoint.x=0;
	}
	ptSplitterBarStartPoint.x+=rectArg.left;
	if (ptSplitterBarStartPoint.y<0) {
		ptBitmapStartPoint.y=ptSplitterBarStartPoint.y*-1;
		ptSplitterBarStartPoint.y=0;
	}
	ptSplitterBarStartPoint.y+=rectArg.top;

	CBitmap &bmp = (IsMouseOnBitmap()) ? m_objHover : m_objBmp;

	CDC memdc;
	memdc.CreateCompatibleDC(pDC);
	HBITMAP hOldBM = (HBITMAP)
		::SelectObject(memdc.GetSafeHdc(), bmp.GetSafeHandle());

	pDC->BitBlt(ptSplitterBarStartPoint.x,ptSplitterBarStartPoint.y,
		szBitmapToDraw.cx, szBitmapToDraw.cy,
		&memdc, ptBitmapStartPoint.x, ptBitmapStartPoint.y,
		SRCCOPY);

	::SelectObject(memdc.GetSafeHdc(),hOldBM);
	
	// set last rectangle of image
	m_rcLastImageRect = CRect(ptSplitterBarStartPoint.x,ptSplitterBarStartPoint.y,
		ptSplitterBarStartPoint.x+szBitmapToDraw.cx, ptSplitterBarStartPoint.y+szBitmapToDraw.cy);

	// from here draw triangle arrows on bar (predefined Win32 arrow icons are ugly so werent used)
	CRgn rgn1,rgn2;
	CBrush br;
	br.CreateSolidBrush((bMouseOnBitmap) ? 
		::GetSysColor(COLOR_3DDKSHADOW) : ::GetSysColor(COLOR_GRAYTEXT));

	CreateArrowRgn(rgn1,m_rcLastImageRect,bVerticalSplitter,!bBarOnEdge,TRUE);
	CreateArrowRgn(rgn2,m_rcLastImageRect,bVerticalSplitter,!bBarOnEdge,FALSE);

	pDC->FillRgn(&rgn1,&br);
	pDC->FillRgn(&rgn2,&br);
}
// virtual calls the drawing functions :  previous ones
void CNSSplitter::OnDrawSplitter(CDC* pDC, ESplitType nType, const CRect& rectArg)
{
	if (!m_bFlatSplitter) {
		CSplitterWnd::OnDrawSplitter(pDC, nType, rectArg);
		DrawSplitterBitmap(pDC, nType, rectArg);
	 	return;
	}

	// draw split by myself
	if((nType != splitBorder) || (pDC == NULL))
	{
		CSplitterWnd::OnDrawSplitter(pDC, nType, rectArg);
		DrawSplitterBitmap(pDC, nType, rectArg);
		return;
	}

	ASSERT_VALID(pDC);

	// Draw border
	pDC->Draw3dRect(rectArg, GetSysColor(COLOR_BTNSHADOW), GetSysColor(COLOR_BTNHIGHLIGHT));
	DrawSplitterBitmap(pDC, nType, rectArg);
}

/* Sets the bitmaps of the splitter the bitmap when splitter
is centered and bitmap when splitter is sided */
BOOL CNSSplitter::SetBitmaps(UINT nRes,UINT nResHover)
{
	if (!nRes && !nResHover) return FALSE;
	BOOL bResult1 = SetBitmap(m_objBmp,nRes);
	BOOL bResult2 = SetBitmap(m_objHover,nResHover);
	return (bResult1 && bResult2);
}
BOOL CNSSplitter::SetBitmap(CBitmap &bmp,UINT nRes)
{
	if (!nRes) return FALSE; // delete previous bitmap
	if (bmp.GetSafeHandle())
		bmp.DeleteObject();
	
	if (bmp.LoadBitmap(nRes)) {
		BITMAP bt;
		bmp.GetBitmap(&bt);
		m_szImage = CSize(bt.bmWidth,bt.bmHeight);
		return TRUE;
	}
	return FALSE;
}

void CNSSplitter::OnLButtonUp(UINT nFlags, CPoint point) 
{
	CSplitterWnd::OnLButtonUp(nFlags, point);

	BOOL bMouseOnBitmap = IsMouseOnBitmap(point);
	if (IsMouseOnBitmap(point))
		SetCustomCursor(TRUE);
	else SetCustomCursor(FALSE);

}

void CNSSplitter::OnMouseMove(UINT nFlags, CPoint point) 
{
	CSplitterWnd::OnMouseMove(nFlags, point);
	
	BOOL bMouseHoversBitmap = IsMouseOnBitmap(point);
	if ((bMouseHoversBitmap && !m_bHoverBitmapDrawn) ||
			(!bMouseHoversBitmap && m_bHoverBitmapDrawn))
	{
		UpdateSplitterBitmap(TRUE);
	}
}

// SetSplitCursor overrided
void CNSSplitter::SetSplitCursor(int ht)
{
	BOOL bMouseOnBitmap = IsMouseOnBitmap();
	if (IsMouseOnBitmap()) {
		SetCustomCursor(TRUE);
		return;
	}
	SetCustomCursor(FALSE);
}


void CNSSplitter::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
	if (!IsMouseOnBitmap(point)) // custom behaviour only on bitmap
	{
		CSplitterWnd::OnLButtonDblClk(nFlags, point);
		return;
	}

	int nPrev = m_nOptimizedSplitterSize;
	CRect rect;
	GetClientRect(&rect);

	if (IsVerticalSplitter())
	{
		while (rect.Width()<m_nOptimizedSplitterSize)
			m_nOptimizedSplitterSize/=2;
		int nSize = (IsBarOnEdge()) ? m_nOptimizedSplitterSize : 0;

		if (nSize>=0)
			TrackColumnSize(nSize,0);
		else {
			nSize = min(rect.Width()+nSize,rect.Width()+m_nOptimizedSplitterSize);
			TrackColumnSize(nSize,0);
		}

	} else { // horizontal splitter

		while (rect.Width()<m_nOptimizedSplitterSize)
			m_nOptimizedSplitterSize/=2;

		int nSize = (IsBarOnEdge()) ? m_nOptimizedSplitterSize : 0;
		if (nSize>=0)
			TrackRowSize(nSize,0);
		else {
			nSize = min(rect.Height()+nSize,rect.Height()+m_nOptimizedSplitterSize);
			TrackRowSize(nSize,0);	
		}
	}
	RecalcLayout();
	m_nOptimizedSplitterSize = nPrev;

	CSplitterWnd::OnLButtonDblClk(nFlags, point);
}

void CNSSplitter::OnDestroy() 
{
	if (nTimerID) KillTimer(nTimerID);
	CSplitterWnd::OnDestroy();
}

void CNSSplitter::OnTimer(UINT nIDEvent) 
{
	if (nIDEvent==nTimerID)
	{
		UpdateSplitterBitmap(FALSE);
	}
	else CSplitterWnd::OnTimer(nIDEvent);
}


